@@ -9,7 +9,7 @@ use Mail::Mbox::MessageParser;
 
 use vars qw( $VERSION $DEBUG $FROM_PATTERN );
 
-$VERSION = '1.4.0';
+$VERSION = '1.03';
 
 *DEBUG = \$Mail::Mbox::MessageParser::DEBUG;
 *FROM_PATTERN = \$Mail::Mbox::MessageParser::FROM_PATTERN;
@@ -35,8 +35,7 @@ sub new
 
   $self->reset();
 
-  $self->{'file_name'} = $options->{'file_name'}
-    if defined $options->{'file_name'};
+  $self->{'file_name'} = $options->{'file_name'};
 
   $self->{'READ_CHUNK_SIZE'} = $READ_CHUNK_SIZE;
 
@@ -98,12 +97,10 @@ sub _IS_A_PIPE
 {
   my $file_handle = shift;
 
-  return (-t $file_handle || -S $file_handle || -p $file_handle ||
-    !-f $file_handle || !(seek $file_handle, 0, 1));
+  return (-t $file_handle || -S $file_handle ||
+     -p $file_handle || !(seek $file_handle, 0, 1));
 }
 
-#-------------------------------------------------------------------------------
-
 sub _read_prologue
 {
   my $self = shift;
@@ -206,9 +203,8 @@ sub read_next_email
     # Keep looking if the header we found is part of a "Begin Included
     # Message".
     my $end_of_string = substr($self->{'READ_BUFFER'}, $self->{'END_OF_EMAIL'}-200, 200);
-    my $endline = $self->{'endline'};
     next if $end_of_string =~
-        /$endline-----(?: Begin Included Message |Original Message)-----$endline[^\r\n]*(?:$endline)*$/i;
+        /\n-----(?: Begin Included Message |Original Message)-----\n[^\n]*\n*$/i;
 
     # Found the next email!
     $self->{'email_length'} = $self->{'END_OF_EMAIL'}-$self->{'START_OF_EMAIL'};
